@@ -65,6 +65,8 @@
 
 #include "message.hpp"
 
+#include "camera.hpp"
+
 // STL
 using namespace std;
 #include <iostream>
@@ -81,6 +83,11 @@ using namespace std;
 
 Config *cfg;
 
+Camera camera;
+Vector zero;
+
+const float ones[] = { 1.0, 1.0, 1.0, 1.0 };
+
 typedef unsigned char uchar;
 
 // Some global variables.... 
@@ -90,7 +97,6 @@ int do_vpswitch = 1;
 
 
 // Our display mode settings.
-int Light_On = 1;
 int Blend_On = 0;
 int Texture_On = 1;
 int Alpha_Add = 1;
@@ -105,7 +111,7 @@ GLint TexModes[] = {GL_DECAL,GL_MODULATE,GL_BLEND,GL_REPLACE};
 long last_time;  // for movement timing
 
 
-// time of last cbrenderscene
+// time of last renderscene
 long last_renderscene_time = 0;
 // time of last keypress 
 float last_keypress = 0;
@@ -128,30 +134,28 @@ VDesktops vdesks;
 
 MessageManager msgmgr;
 
-
-// lights
-
-// diffuse light
-//float light_ambient[]=  { 1.0f, 1.0f, 1.0f, 1.0f };
-float light_diffuse[]=  { 1.0f, 1.0f, 1.0f, 1.0f }; 
-float light_position[]= { 0.0f, 0.0f, 17.0f, 1.0f };
-
-
 Movement digit_vis;
 float digit_visibility = 0.0;
 
-Movement breathing;  // changes the ambient light to look like breathing
+Movement face_vis;
+float face_visibility = 0.0;
+
+// LIGHTS
+//#define DEBUG_LIGHTING
+
+float light_position[]= { 0.0f, 0.0f, 17.0f, 1.0f };
+
+// diffuse light to light the front face
+float light_diffuse[]=  { 1.0f, 1.0f, 1.0f, 1.0f }; 
+
+// ambient light for a subtle background breathing effect
+float light_ambient[]=  { 1.0f, 1.0f, 1.0f, 1.0f };
+
+// changes the ambient light to look like breathing
+Movement breathing;
 float ambient_value = 2.0;
 
 
-// ambient light
-float light_ambient2[]=  { 1.0f, 1.0f, 1.0f, 1.0f };
-//float light_diffuse2[]=  { 1.0f, 1.0f, 1.0f, 1.0f }; 
-float light_position2[]= { 0.0f, 0.0f, 17.0f, 1.0f };
-
-
-
-
 // this contains all the "Face" types and each holds location
 // information and texture_id.  This in turn is manipulated by each
 // faceset (the coords of each face not the texture_id).  This things
@@ -209,6 +213,8 @@ static int main_loop(void);
 static void gl_init (void);
 static void initialize_3ddesktop (void);
 static void load_digits(void);
+static void load_background_image(void);
+static int background_image_is_new(char *bg_image_filename);
 
 
 // Some Event hooks
@@ -272,6 +278,8 @@ entry_move_start_hook (Event *e)
 
     digit_vis.init(&digit_visibility, 0.0, 1.0, 45, Movement::MOVE_TYPE_SMOOTH);
 
+    face_vis.init(&face_visibility, 1.0, 0.7, 45, Movement::MOVE_TYPE_SMOOTH);
+
     faces->entry_movement_start();
 }
 
@@ -283,6 +291,8 @@ exit_move_start_hook (Event *e)
 
     digit_vis.init(&digit_visibility, 1.0, 0.0, 45, Movement::MOVE_TYPE_SMOOTH);
 
+    face_vis.init(&face_visibility, 0.7, 1.0, 45, Movement::MOVE_TYPE_SMOOTH);
+
     faces->exit_movement_start();
 }
 
@@ -424,9 +434,36 @@ grab_portion_of_screen (portion_control_t *pc)
 
 
 
+// ------
+#define FRAME_RATE_SAMPLES 100   // at 100 fps this will be about every second we'll get a value
+int   framecount = 0;
+float framerate = 0;
+
+static void 
+calc_fps(void)
+{
+    static long last = 0;
+    long now;
+    double delta; // the number of seconds that we took to draw FRAME_RATE_SAMPLES frames
+
+    if (++framecount >= FRAME_RATE_SAMPLES) {
+        now = get_milli_time();
+        delta = (double)(now - last) / (double)1000.0;
+        //now  = clock();
+        //delta = (now - last) / (float) CLOCKS_PER_SEC;
+        last = now;
+
+        framerate = (float) FRAME_RATE_SAMPLES / delta;
+        framecount = 0;
+        msgout(DEBUG, "FPS: %lf\n", framerate);
+    }
+} // END calc_fps
+
+
+#if 0
 // ------
 // Frames per second (FPS) statistic variables and routine.
-#define FRAME_RATE_SAMPLES 100
+#define FRAME_RATE_SAMPLES 50
 int   FrameCount = 0;
 float FrameRate = 0;
 
@@ -444,9 +481,10 @@ calc_fps(void)
 
         FrameRate = FRAME_RATE_SAMPLES / delta;
         FrameCount = 0;
+        msgout(DEBUG, "FPS: %lf\n", FrameRate);
     }
 } // END calc_fps
-
+#endif
 
 uint digit_texture;
 
@@ -549,14 +587,7 @@ draw_x (float x_trans, float y_trans)
 
     float w = cfg->options->digit_size/2.0;
 
-    // But we like our current view too; so we save it here.
-    glPushMatrix();
-    
-    // Now we set up a new projection for the text.
-    glLoadIdentity();
-    glOrtho(0,GLWin.get_width(),0,GLWin.get_height(),-1.0,1.0);
-
-
+    // assume we are already in projection matrix
     glTranslatef(x_trans, y_trans, 0.0f);
 
     set_color (cfg->options->digit_color, digit_visibility);
@@ -582,9 +613,6 @@ draw_x (float x_trans, float y_trans)
     glVertex3f(-w,w,0);
     
     glEnd();
-
-    // Done with this special projection matrix.  Throw it away.
-    glPopMatrix();
 }
 
 
@@ -622,17 +650,9 @@ draw_digit (int d, float x_trans, float y_trans)
 
     float w = cfg->options->digit_size/2.0;
 
-    // But we like our current view too; so we save it here.
-    glPushMatrix();
-    
-    // Now we set up a new projection for the text.
-    glLoadIdentity();
-    glOrtho(0,GLWin.get_width(),0,GLWin.get_height(),-1.0,1.0);
-
-
+    // assume we are in projection matrix already
     glTranslatef(x_trans, y_trans, 0.0f);
 
-
     set_color (cfg->options->digit_color, digit_visibility);
 
     glEnable(GL_BLEND);
@@ -657,10 +677,152 @@ draw_digit (int d, float x_trans, float y_trans)
     
     glEnd();
 
-    // Done with this special projection matrix.  Throw it away.
-    glPopMatrix();
 }
 
+static bool
+allow_blending(void)
+{
+    return (faces->allow_blending() && Blend_On);
+}
+
+
+static void 
+setup_gl_modes(void)
+{
+    if (Texture_On)
+        glEnable(GL_TEXTURE_2D);
+    else
+        glDisable(GL_TEXTURE_2D);
+
+    glEnable(GL_LIGHTING);
+
+    // If we're blending, we don't want z-buffering.
+    if (allow_blending()) {
+        glDisable(GL_DEPTH_TEST); 
+        glEnable(GL_BLEND);
+    } else { 
+        glEnable(GL_DEPTH_TEST); 
+        glDisable(GL_BLEND); 
+    }
+
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+    glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,TexModes[Curr_TexMode]);
+
+    // only effects gl_blend
+    //float text_env_color[] = { .1, .1, .1, .1 };
+    //glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR, text_env_color);
+
+}  // setup_gl_modes
+
+
+uint background_texture;
+
+static void
+draw_background_image(void)
+{
+    float screen_width = GLWin.get_width();
+    float screen_height = GLWin.get_height();
+        
+    glMatrixMode(GL_PROJECTION);
+
+    glPushMatrix();
+
+    glLoadIdentity();
+
+    glOrtho(0,screen_width,0,screen_height,-1.0,1.0);
+
+    glMatrixMode(GL_MODELVIEW);
+
+    glLoadIdentity();
+
+    glEnable(GL_TEXTURE_2D);
+    glDisable(GL_LIGHTING);
+
+    glDisable(GL_DEPTH_TEST); 
+    glDepthMask(GL_FALSE);
+
+    //glTranslatef(0.0f, 0.0f, 0.0f);
+        
+    // leave it up to the creator of the image to keep it from being too bright
+    //glColor4f(.5,.5,.5,.5);
+    glColor4f(1.0,1.0,1.0,1.0);
+
+    glBindTexture(GL_TEXTURE_2D, background_texture);
+
+    glBegin(GL_QUADS);
+     glTexCoord2f (0.0f, 1.0f);
+     glVertex3f(        0.0f,          0.0f, 0.0f);
+
+     glTexCoord2f (0.0f, 0.0f);
+     glVertex3f(        0.0f, screen_height, 0.0f);
+
+     glTexCoord2f (1.0f, 0.0f);
+     glVertex3f(screen_width, screen_height, 0.0f);
+
+     glTexCoord2f (1.0f, 1.0f);
+     glVertex3f(screen_width,          0.0f, 0.0f);
+    glEnd();
+
+    glMatrixMode(GL_PROJECTION);    
+
+    glPopMatrix();
+
+    glMatrixMode(GL_MODELVIEW);
+
+    glEnable(GL_TEXTURE_2D);
+    glEnable(GL_LIGHTING);
+    glEnable(GL_DEPTH_TEST);
+    glDepthMask(GL_TRUE);
+
+} // draw_background_image
+
+
+
+//
+// Some hacky performance measurement stuff
+
+//#define PERF_DEBUG 1
+#if defined PERF_DEBUG
+
+struct timeit_data {
+    long usecs;
+    long count;
+    long total;
+};
+
+#define NUM_TIMEIT_SLOTS 20
+struct timeit_data timeit[NUM_TIMEIT_SLOTS];
+
+#define TIMEIT_INIT()       do { memset(timeit, 0, sizeof(struct timeit_data) * NUM_TIMEIT_SLOTS); } while (0);
+#define TIMEIT_START(i)      do {  \
+                               struct timeval tv; \
+                               gettimeofday(&tv, 0); \
+                               timeit[(i)].usecs = tv.tv_sec * 1000000 + tv.tv_usec; \
+                             } while (0);
+#define TIMEIT_STOP(i)       do { \
+                               struct timeval tv; \
+                               gettimeofday(&tv, 0); \
+                               long delta = (tv.tv_sec * 1000000 + tv.tv_usec)  -  timeit[(i)].usecs; \
+                               if ((timeit[(i)].count < 10) || (delta < 2 *(timeit[(i)].total / timeit[(i)].count))) { \
+                                   timeit[(i)].total += delta; /* this smooths out the running average a bit */  \
+                               } \
+                               timeit[(i)].count++;           \
+                             } while (0);
+
+#define TIMEIT_PRINT(i, s)   do {                                      \
+                               printf("Running average for %s(%d): %lf useconds (%ld samples)\n", \
+                                      (s), (i), (double)(timeit[(i)].total) / (double)(timeit[(i)].count), timeit[(i)].count); \
+                             } while(0);
+
+#else 
+
+#define TIMEIT_INIT()
+#define TIMEIT_START(i)
+#define TIMEIT_STOP(i)
+#define TIMEIT_PRINT(i,s)
+
+#endif // if defined PERF_DEBUG
 
 // ------
 // Routine which actually does the drawing
@@ -670,61 +832,6 @@ render_scene(void)
     //char buf[80]; // For our strings.
     long current_time;
 
-
-    // Enables, disables or otherwise adjusts as 
-    // appropriate for our current settings.
-
-    if (Texture_On)
-        glEnable(GL_TEXTURE_2D);
-    else
-        glDisable(GL_TEXTURE_2D);
-
-    if (Light_On) 
-        glEnable(GL_LIGHTING);
-    else 
-        glDisable(GL_LIGHTING);
-
-    if (Alpha_Add)
-        //glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-
-        //glBlendFunc(GL_ONE, GL_SRC_ALPHA); 
-
-        //glBlendFunc(GL_ONE, GL_CONSTANT_ALPHA); 
-        
-        // this one is good with NO color material, disable depth test, blend on
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE); // GOOD!
-
-    else
-        glBlendFunc(GL_ONE, GL_ONE_MINUS_DST_ALPHA);
-    
-    //glBlendFunc(GL_ONE_MINUS_DST_ALPHA,GL_SRC_ALPHA); 
-
-
-    // If we're blending, we don't want z-buffering.
-    if (Blend_On) {
-        //glEnable(GL_DEPTH_TEST); 
-        glDisable(GL_DEPTH_TEST); 
-        glEnable(GL_BLEND);
-    } else { 
-        glEnable(GL_DEPTH_TEST); 
-        glDisable(GL_BLEND); 
-        //glEnable(GL_DEPTH_TEST); 
-    }
-
-    glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,TexModes[Curr_TexMode]);
-
-    // only effects gl_blend
-    //float text_env_color[] = { .1, .1, .1, .1 };
-    //glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR, text_env_color);
-
-
-    // Need to manipulate the ModelView matrix to move our model around.
-    glMatrixMode(GL_MODELVIEW);
-
-    // Reset to 0,0,0; no rotation, no scaling.
-    glLoadIdentity(); 
-
-
     current_time = get_milli_time();
 
     // this gives us semi-precise/consistent timing of the movements
@@ -734,8 +841,6 @@ render_scene(void)
         if (ISSET_STATE(STATE_EXIT)) {
 
             //if (!cfg->options->entry_exit_movement || faces->is_exit_movement_done()) {
-            
-
 
             if (faces->is_exit_movement_done()) {
                 // quit (the z offset was changed to match the screen)
@@ -780,7 +885,7 @@ render_scene(void)
                 breathing.init(&ambient_value, 1.7, 3.0, 200, Movement::MOVE_TYPE_SINE);
             } else {
                 breathing.change_a_bit();
-                light_ambient2[0] = light_ambient2[1] = light_ambient2[2] = ambient_value;
+                light_ambient[0] = light_ambient[1] = light_ambient[2] = ambient_value;
             }
 
         }
@@ -788,6 +893,10 @@ render_scene(void)
         if (!digit_vis.at_destination())
             digit_vis.change_a_bit();
 
+        if (!face_vis.at_destination())
+            face_vis.change_a_bit();
+
+        faces->set_transparency(face_visibility);
 
         faces->do_movement();
 
@@ -795,7 +904,6 @@ render_scene(void)
 
     } // end animation step
  
-
     // FIXME: use event to accomplish this?
 
     // after some seconds start some fun ;)
@@ -812,108 +920,90 @@ render_scene(void)
         }
     }
 
+    setup_gl_modes();
+
+    glClear(GL_COLOR_BUFFER_BIT | (allow_blending() ? 0 : GL_DEPTH_BUFFER_BIT));
+
+    if (cfg->use_background_image) {
+
+        draw_background_image();
+
+        setup_gl_modes();  // have to do this again as draw_bg_image changed it
+    }
+
+    glMatrixMode(GL_MODELVIEW);
+
+    glPushMatrix();
+
     glLoadIdentity();
 
-    // adjust the faces the necessary amount
-    faces->do_position_GL();
+    // zero position of camera (FIX THIS!)
+    camera.position = zero;
+    camera.viewvector = zero;
 
-    // Clear the color and depth buffers.
-    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+    // adjust the faces the necessary amount
+    faces->position_faces();
 
     faces->render();
 
-    //glPushMatrix();
-
-    glLoadIdentity();
-
-    glColor3f(0.0, 0.0, 1.0);
-    glPointSize(5.0);
-    glBegin(GL_POINTS);
-    glVertex4fv(light_position2);
-    glVertex4fv(light_position);
-    glEnd();
-
-
-    // lighting (light2 is for ambient only)
-    glLightfv(GL_LIGHT2, GL_POSITION, light_position2);
-    glLightfv(GL_LIGHT2, GL_AMBIENT,  light_ambient2);
-    //glLightfv(GL_LIGHT2, GL_DIFFUSE,  Light_Diffuse2); 
-    glEnable (GL_LIGHT2);
-
-    // lighting (light1 is diffuse only)
-    glLightfv(GL_LIGHT1, GL_POSITION, light_position);
-
-    //glLightfv(GL_LIGHT1, GL_AMBIENT,  Light_Ambient);
-    //glLightfv(GL_LIGHT1, GL_DIFFUSE,  light_diffuse);
-
-    //float halfs[] = { .50, .50, .50, .50 };
-    float ones[] = { 1.0, 1.0, 1.0, 1.0 };
-
-
-    glLightfv(GL_LIGHT1, GL_DIFFUSE,  ones);
-    glLightfv(GL_LIGHT1, GL_SPECULAR,  ones);
-
-    glEnable (GL_LIGHT1);
-
-
-    //glPopMatrix();
-
-    // We need to change the projection matrix for the text rendering.  
-    glMatrixMode(GL_PROJECTION);
-
-    // But we like our current view too; so we save it here.
-    glPushMatrix();
-
-    // Now we set up a new projection for the text.
-    glLoadIdentity();
-    glOrtho(0,GLWin.get_width(),0,GLWin.get_height(),-1.0,1.0);
-
-    // Lit or textured text looks awful.
-    glDisable(GL_TEXTURE_2D);
-    glDisable(GL_LIGHTING);
-
-    // We don't want depth-testing either.
-    glDisable(GL_DEPTH_TEST); 
-
-
-    // Now we want to render the calulated FPS at the top.
-   
-    // To ease, simply translate up.  Note we're working in screen
-    // pixels in this projection.
-   
-#if 0
-    if (cfg->options->show_fps) {
-        glTranslatef(6.0f, GLWin.get_height() - 14,0.0f);
-        
-        // Make sure we can read the FPS section by first placing a 
-        // dark, mostly opaque backdrop rectangle.
-        glColor4f(0.2,0.2,0.2,0.75);
-        
-        glBegin(GL_QUADS);
-        glVertex3f(  0.0f, -2.0f, 0.0f);
-        glVertex3f(  0.0f, 12.0f, 0.0f);
-        glVertex3f(140.0f, 12.0f, 0.0f);
-        glVertex3f(140.0f, -2.0f, 0.0f);
-        glEnd();
-        
-        glColor4f(0.9,0.2,0.2,.75);
-        char buf[50];
-        sprintf(buf,"FPS: %f F: %2d", FrameRate, FrameCount);
-        glRasterPos2i(6,0);
-        ourPrintString(GLUT_BITMAP_HELVETICA_12,buf);
-    }
-#endif // if 0
-
-    // Done with this special projection matrix.  Throw it away.
     glPopMatrix();
 
+    //
+    //  Now do some lighting
+    // 
+    glPushMatrix();
 
+    glLoadIdentity();
+
+    faces->position_lighting();
+
+
+// for debugging
+#if defined DEBUG_LIGHTING
+    glColor3f(1.0, 1.0, 1.0);
+    glPointSize(15.0);
+    glBegin(GL_POINTS);
+    glVertex3f(0, 0, 0);
+    glEnd();
+
+    glColor3f(1.0, 0.0, 0.0);
+    glPointSize(15.0);
+    glBegin(GL_POINTS);
+    glVertex4fv(light_position);
+    glEnd();
+#endif
+
+    // Lighting 
+
+    // LIGHT1 is a diffuse light on the front face
+    glLightfv(GL_LIGHT1, GL_POSITION, light_position);
+    glLightfv(GL_LIGHT1, GL_DIFFUSE,  ones);
+    glLightfv(GL_LIGHT1, GL_SPECULAR,  ones);
+    glEnable (GL_LIGHT1);
+
+    // LIGHT2 is ambient background light
+    glLightfv(GL_LIGHT2, GL_POSITION, light_position);
+    glLightfv(GL_LIGHT2, GL_AMBIENT,  light_ambient);
+    glEnable (GL_LIGHT2);
+
+
+    glPopMatrix();
 
     if (cfg->options->show_digits) {
 
         // We need to change the projection matrix for the text rendering.  
         glMatrixMode(GL_PROJECTION);
 
+        glPushMatrix();
+
+        glLoadIdentity();
+
+        glOrtho(0,GLWin.get_width(),0,GLWin.get_height(),-1.0,1.0);
+
+        glMatrixMode(GL_MODELVIEW);
+
+        glLoadIdentity();
+
         // Lit or textured text looks awful.
         glEnable(GL_TEXTURE_2D);
         glDisable(GL_LIGHTING);
@@ -971,13 +1061,15 @@ render_scene(void)
         glRasterPos2i(6,0);
         ourPrintString(GLUT_BITMAP_TIMES_ROMAN_24,buf);
 #endif
-        
+
+        glMatrixMode(GL_PROJECTION);
+
+        glPopMatrix();
+
+        glMatrixMode(GL_MODELVIEW);
 
     }
 
-
-
-    // All done drawing.  Let's show it.
     glXSwapBuffers(GLWin.get_display(), GLWin.get_window());
 
     // And collect our statistics.
@@ -997,6 +1089,31 @@ mouse_pressed(unsigned int button)
 
     //msgout (DEBUG, "button pressed %d - 0x%x\n", button, button);
 
+    list<Config::mousebinding *>::iterator k;
+    for (k = cfg->mousebindings.begin(); k != cfg->mousebindings.end(); k++) {
+        if ((*k)->button == button) {
+            switch ((*k)->action) {
+            case Config::LEFT:
+                faces->goto_left();
+                break;
+            case Config::RIGHT:
+                faces->goto_right();
+                break;
+            case Config::DOWN:
+                faces->goto_down();
+                break;
+            case Config::UP:
+                faces->goto_up();
+                break;
+            case Config::SELECT:
+                begin_exit();
+                break;
+            }
+            return;
+        }
+    }
+
+
     switch (button) {
     case 1: // left
     	if (cfg->options->alt_mousebuttons)
@@ -1060,6 +1177,7 @@ mouse_pressed(unsigned int button)
         break;
 
     }
+
 } // END mouse_pressed
 
 
@@ -1080,10 +1198,34 @@ key_pressed(KeySym key)
 
     next_action = 0;
 
+    list<Config::keybinding *>::iterator k;
+    for (k = cfg->keybindings.begin(); k != cfg->keybindings.end(); k++) {
+        if ((*k)->key == key) {
+            switch ((*k)->action) {
+            case Config::LEFT:
+                faces->goto_left();
+                break;
+            case Config::RIGHT:
+                faces->goto_right();
+                break;
+            case Config::DOWN:
+                faces->goto_down();
+                break;
+            case Config::UP:
+                faces->goto_up();
+                break;
+            case Config::SELECT:
+                begin_exit();
+                break;
+            }
+            return;
+        }
+    }
+
     switch (key) {
     case 130: case 98: // B - Blending.
         Blend_On = Blend_On ? 0 : 1; 
-        if (!Blend_On)
+        if (!allow_blending())
             glDisable(GL_BLEND); 
         else
             glEnable(GL_BLEND);
@@ -1092,9 +1234,6 @@ key_pressed(KeySym key)
     case 108: case 76:  // L - last desktop
         SET_STATE(STATE_GOTO);
         faces->goto_face(last_coords);
-        
-        // this used to be for turning lights on an off...
-        //Light_On = Light_On ? 0 : 1; 
         break;
 
     case 109: case 77:  // M - Mode of Blending
@@ -1121,7 +1260,8 @@ key_pressed(KeySym key)
         disable_random_fun = disable_random_fun ? 0 : 1;
         break;
 
-#if 1 // for debugging best light position...
+// for debugging best light position...
+#if defined DEBUG_LIGHTING 
     case 119: // w
         light_position[0] -= 0.5;
         msgout (DEBUG, "light x at %f\n", light_position[0]);
@@ -1240,6 +1380,7 @@ key_pressed(KeySym key)
         msgout (DEBUG, "KP: No action for %ld / 0x%lx.\n", key, key);
         break;
     }
+
 } // END key_pressed
 
 
@@ -1314,23 +1455,23 @@ demand_load_arrangement (FaceType_t type)
         
         switch (type) {
         case FACE_TYPE_CAROUSEL:
-            f = new CarouselArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new CarouselArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
 #if !defined (ONE_ARRANGEMENT_ONLY)
         case FACE_TYPE_CYLINDER:
-            f = new CylinderArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new CylinderArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
         case FACE_TYPE_VIEWMASTER:
-            f = new ViewMasterArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new ViewMasterArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
         case FACE_TYPE_PRICEISRIGHT:
-            f = new PriceIsRightArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new PriceIsRightArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
         case FACE_TYPE_LINEAR:
-            f = new LinearArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new LinearArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
         case FACE_TYPE_FLIP:
-            f = new FlipArrangement (vdesks, face_width, face_height, face_set, cfg);
+            f = new FlipArrangement (vdesks, face_width, face_height, face_set, cfg, &camera);
             break;
 #endif
         default:
@@ -1511,7 +1652,7 @@ wait_for_activation(void)
         switch (msgmgr.msg.mtype) {
         case MTYPE_STOP:
             msgout (DEBUG, "-- quiting\n");
-
+            TIMEIT_PRINT(1, "render_scene    ");
             end_program(0);
 
             break;
@@ -1742,6 +1883,11 @@ wait_for_activation(void)
 
             }
 
+            // (re)load the background image if the one in the cfg is new
+            if (cfg->use_background_image && background_image_is_new(cfg->default_background)) {
+                load_background_image();
+            }
+
             done = 1;
             break;
         default:
@@ -1788,7 +1934,7 @@ idle_func(void)
 
     long start = get_milli_time();
 
-    if ( (start - last_renderscene_time) >= 1 ) {
+    if ( (start - last_renderscene_time) >= 10 ) {
 
         //msgout (DEBUG, "diff = %lu\n", start - last_renderscene_time);
 
@@ -1799,7 +1945,6 @@ idle_func(void)
         context_switch();
     }
 
-
 } // END idle_func
 #endif
 
@@ -1949,8 +2094,8 @@ main_loop(void)
                     break;
                 }
             }
-
-            if (cfg->options->use_context_switch)
+            
+            if (cfg->use_context_switch)
                 idle_func();
             else
                 render_scene();
@@ -1983,25 +2128,19 @@ gl_init (void)
     // Enables Smooth Color Shading; try GL_FLAT for (lack of) fun.
     glShadeModel(GL_SMOOTH);
 
-    glLoadIdentity();
+    camera.PositionCamera(0, 0, 0,   // position
+                          0, 0, 1,   // view
+                          0, 1, 0 ); // up vector
 
-    // A handy trick -- have surface material mirror the color.
-    //glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
-    //glEnable(GL_COLOR_MATERIAL);
 
-    //glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 
     //glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
 
-    float ones[] = { 1.0, 1.0, 1.0, 1.0 };
-    //float halfs[] = { .50, .50, .50, .50 };
-
     glMaterialfv(GL_FRONT, GL_DIFFUSE, ones);
     glMaterialfv(GL_FRONT, GL_SPECULAR, ones);
     glMaterialf(GL_FRONT, GL_SHININESS, 100.0);
 
-
 #if 0
     float spot_direction[] = { 0.0, 0.0, -1.0 };
     glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, spot_direction);
@@ -2009,11 +2148,12 @@ gl_init (void)
     glLighti(GL_LIGHT1, GL_SPOT_CUTOFF, 180);
 #endif
 
-
     if (cfg->options->use_breathing)
         breathing.init(&ambient_value, 1.7, 3.0, 200, Movement::MOVE_TYPE_SINE);
     else
-        light_ambient2[0] = light_ambient2[1] = light_ambient2[2] = 2.0;
+        light_ambient[0] = light_ambient[1] = light_ambient[2] = 2.0;
+
+    
 
 } // END gl_init
 
@@ -2025,7 +2165,7 @@ load_digits(void)
         
     char filename[100];
     unsigned char *data;
-    unsigned int sizex, sizey;
+    unsigned int sizex = 256, sizey = 256;
     
     extern Config *cfg;
 
@@ -2033,7 +2173,7 @@ load_digits(void)
 
 
     // allocate space for texture
-    data = new unsigned char [256 * 256 * 4];
+    data = new unsigned char [sizex * sizey * 4];
     if (data == NULL) {
         msgout(ERROR, "Error allocating space for image");
         end_program(0);
@@ -2084,12 +2224,98 @@ load_digits(void)
 
 } // END load_digits
 
+#if defined(__FreeBSD__)
+static char background_img_in_use[MAXNAMLEN + 1];
+#else
+static char background_img_in_use[PATH_MAX];
+#endif
+
+
+static int
+background_image_is_new(char *bg_image_filename) 
+{
+    return strcmp(bg_image_filename, background_img_in_use);
+}
+
+static void 
+load_background_image(void) 
+{
+        
+    char filename[100];
+    unsigned char *data;
+    unsigned int bg_img_size = cfg->bg_texture_size;
+    
+    extern Config *cfg;
+
+    // allocate space for texture
+    data = new unsigned char [bg_img_size * bg_img_size * 4];
+    if (data == NULL) {
+        msgout(ERROR, "Error allocating space for image");
+        end_program(0);
+    }
+        
+    // try loading /usr/share/3ddesktop/x
+    //             /usr/local/share/3ddesktop/x
+    //             ~/.3ddesktop/x
+    //             ./x
+
+    Imlib_Image bg_img;
+
+    sprintf (filename, "%s", cfg->default_background);
+
+    if (! (bg_img = imlib_load_image(filename)) ) {
+
+        sprintf (filename, SHAREDIR "/%s", cfg->default_background);
+
+        if (! (bg_img = imlib_load_image(filename)) ) {
+
+            sprintf (filename, "%s/%s", cfg->working_dir, 
+                     cfg->default_background);
+            
+            if (! (bg_img = imlib_load_image(filename)) ) {
+                
+                sprintf (filename, "./%s", cfg->default_background);
+                
+                if (! (bg_img = imlib_load_image(filename)) ) {
+                    msgout (DEBUG, "background image: %s not found\n", 
+                            cfg->default_background);
+                    cfg->use_background_image = 0;
+                    return ;
+                }
+            }
+        }
+    }
+
+    strcpy(background_img_in_use, cfg->default_background);
+
+    convert_imlib_image_to_opengl_data(bg_img_size, bg_img, data);
+
+    imlib_context_set_image(bg_img);
+    imlib_free_image();
+
+    // Create Texture
+    glGenTextures(1, &background_texture );
+    glBindTexture(GL_TEXTURE_2D, background_texture );   // 2d texture (x and y size)
+        
+    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, MIN_FILTER);
+    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, MAX_FILTER);
+        
+    // 2d texture, 3 components (red, green, blue), x size from image, y size from image, 
+    // rgb color data, unsigned byte data, and finally the data itself.
+    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, bg_img_size, bg_img_size, GL_RGBA, GL_UNSIGNED_BYTE, data);
+
+    //glTexImage2D(GL_TEXTURE_2D, 0, 3, sizex, sizey, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
+        
+    delete data;
+
+} // END load_background_image
 
 
 // accesses global cfg
 static void
 initialize_3ddesktop (void)
 {
+    TIMEIT_INIT();
 
     // creates ~/.3ddesktop or /tmp/.3ddesktop-user if not alread there
     // used for pid file but no longer the screenshot
@@ -2216,26 +2442,35 @@ initialize_3ddesktop (void)
 
     set_arrangement(cfg->options->default_z_offset_distance);
 
-    // loop through desktops and capture their images...
-    if (cfg->options->acquire) {
+    // loop through desktops and capture their images...  
+    // ... or, if mirror, don't switch or capture desktops, just grab
+    // current and set to all
+    if (cfg->options->acquire || cfg->mirror) {
 
-        sleep(1);
+        if (!cfg->mirror)
+            sleep(1);
 
+        int firsttime = 1;
         int i, j;
         for (i = 0; i < row_count; i++) {
 
             for (j = 0; j < col_count; j++) {
 
-                vdesks.set_vdesktop (j, i);
+                if (!cfg->mirror) {
+                    vdesks.set_vdesktop (j, i);
 
-                if (cfg->options->acquire > 1) {
-                    msgout(DEBUG, "sleeping %d...\n", cfg->options->acquire);
-                    sleep_ms (cfg->options->acquire);
+                    if (cfg->options->acquire > 1) {
+                        msgout(DEBUG, "sleeping %d...\n", cfg->options->acquire);
+                        sleep_ms (cfg->options->acquire);
+                    }
                 }
                 
-                if (GLWin.grab_screenshot_data() < 0) {
-                    msgout (ERROR, "couldn't grab screen image\n");
-                    end_program(-1);
+                if (firsttime || !cfg->mirror) {
+                    if (GLWin.grab_screenshot_data() < 0) {
+                        msgout (ERROR, "couldn't grab screen image\n");
+                        end_program(-1);
+                    }
+                    firsttime = 0;
                 }
                 
                 face_set->load_texture_data(j + (i * col_count), 
@@ -2245,12 +2480,16 @@ initialize_3ddesktop (void)
         }
 
         // go back to current
-        vdesks.set_vdesktop (current_col, current_row);
+        if (!cfg->mirror)
+            vdesks.set_vdesktop (current_col, current_row);
 
     }
 
     load_digits();
 
+    if (cfg->use_background_image)
+        load_background_image();
+
     if (flag_that_i_am_running()) {
         // (if we've daemonized this goes to syslog which is not easily seen)
         msgout(ERROR, "Oops, someone beat us to start up!!  Amazing!\n");
