@@ -310,9 +310,17 @@ void Arrangement::set_transparency (float t)
 
 void Arrangement::render()
 {
-    glCallList(face_list);
+    face_set->render_all(current_face);
+    //face_set->render_all();
 }
 
+// sacrifice the call list for the pretty transparency... :/
+
+//void Arrangement::render()
+//{
+//    glCallList(face_list);
+//}
+
 void Arrangement::make_display_list(void)
 {
     glMatrixMode(GL_MODELVIEW);
@@ -324,6 +332,20 @@ void Arrangement::make_display_list(void)
     glEndList();
 }
 
+void Arrangement::position_lighting(void)
+{
+    // For most arrangements (carousel, cylinder, priceisright,
+    // viewmaster) we want to move the camera out away from the faces
+    // but not rotate around to the current face (goto_face_val) as
+    // that would spin the lighting around.
+    camera->Look();
+}
+
+
+int Arrangement::allow_blending(void)
+{
+    return true;
+}
 
 // ==============================================================
 //  CarouselArrangement
@@ -332,7 +354,8 @@ void Arrangement::make_display_list(void)
 CarouselArrangement::CarouselArrangement(VDesktops &vdesks, 
 					 float fw, float fh, 
 					 FaceSet *fs,
-					 Config *c)
+					 Config *c,
+                                         Camera *cam)
 					 
     : Arrangement (fs, c)
 {
@@ -341,7 +364,8 @@ CarouselArrangement::CarouselArrangement(VDesktops &vdesks,
 
     face_width = fw;
     face_height = fh;
-                
+
+    camera = cam;
 
     calculate_faces(vdesks);
 
@@ -366,20 +390,30 @@ void CarouselArrangement::calculate_faces(VDesktops & vdesks)
     
     goto_face_val = 360 - (side_angle * current_face);
     
+    // make this value up - give us space between each face
     float circum = face_width * 1.5 * n_faces;
+    
+    // this is a class value that we keep
     radius = circum / (2.0 * PI);
 
+    // angle in radians of right triangle angle from camera to face
     float my_angle = ((FOV / 2.0) * PI) / 180.0;
     
-    // get_base_z_offset returns radius
-    z_offset_close = z_offset = -(float)((face_height/2.0) / tan(my_angle) + 
-                                          get_base_z_offset());
-    
-
-    int i;
     float half_face_height = face_height / 2.0;
     float half_face_width = face_width / 2.0;
 
+    // find distance from camera to face to show whole face then add
+    // radius to it. get_base_z_offset returns radius
+    z_offset_close = z_offset = -(float)(half_face_height / tan(my_angle) + 
+                                         get_base_z_offset());
+    
+
+//    msgout (DEBUG, "_________ rad %f  my_angle %f  circum %f  z_off %f  z_off_c %f  fw %f  fh %f\n", 
+//            radius, my_angle, circum, z_offset, z_offset_close, 
+//            face_width, face_height);
+
+    int i;
+
     Point tl,tr,bl,br;
 
     for (i = 0; i < n_faces; i++) {
@@ -401,7 +435,7 @@ void CarouselArrangement::calculate_faces(VDesktops & vdesks)
         translate (br.v, 0, 0, radius);
         rotate_y (br.v, side_angle * i);
 
-        //msgout (DEBUG"[%i] = %f, %f, %f\n", i, tl.v[0], tl.v[1], tl.v[2]);
+//        msgout (DEBUG, "[%i] = %f, %f, %f\n", i, tl.v[0], tl.v[1], tl.v[2]);
 
 
         // the palindromic serendipity of this rocks!
@@ -409,11 +443,11 @@ void CarouselArrangement::calculate_faces(VDesktops & vdesks)
 
 #if 0
         msgout(DEBUG, "[%i] X tl=%f, tr=%f, br=%f\n", i, 
-                f->topleft.v[0], f->topright.v[0], f->bottomright.v[0]);
+                face_set->topleft.v[0], face_set->topright.v[0], face_set->bottomright.v[0]);
         msgout(DEBUG, "[%i] Y tl=%f, tr=%f, br=%f\n", i, 
-                f->topleft.v[1], f->topright.v[1], f->bottomright.v[1]);
+                face_set->topleft.v[1], face_set->topright.v[1], face_set->bottomright.v[1]);
         msgout(DEBUG, "[%i] Z tl=%f, tr=%f, br=%f\n", i, 
-                f->topleft.v[2], f->topright.v[2], f->bottomright.v[2]);
+                face_set->topleft.v[2], face_set->topright.v[2], face_set->bottomright.v[2]);
 
         msgout(DEBUG, "[%i] normal = %f, %f, %f\n\n", i, 
                 f->normal.v[0], f->normal.v[1], f->normal.v[2]);
@@ -421,7 +455,7 @@ void CarouselArrangement::calculate_faces(VDesktops & vdesks)
 
     }
 
-    make_display_list();
+    //make_display_list();
 }
 
 int CarouselArrangement::get_type () 
@@ -429,80 +463,6 @@ int CarouselArrangement::get_type ()
     return FACE_TYPE_CAROUSEL;
 }
 
-#if 0
-
-// check bounds on i
-static void check(int *i, int max) 
-{
-    if (*i == max + 1) 
-        *i = 1;
-
-    if (*i == 0)
-        *i = max;
-}
-
-
-void CarouselArrangement::render()
-{
-    //for (i = 0; i < n_columns * n_rows; i++)
-    //    faces[i]->render();
-
-    stack<int> nums;
-
-    int max = n_columns * n_rows;
-
-    int c = current_face + 1;;
-
-    //faces[c-1]->render();
-    nums.push(c);
-
-    int l = c - 1;
-    check(&l, max);
-
-    if (l == c)
-        return;
-
-    //faces[l-1]->render();
-    nums.push(l);
-
-    int r = c + 1;
-    check(&r, max);
-
-    if (r == l)
-        return;
-
-    //faces[r-1]->render();
-    nums.push(r);
-
-    while (1) {
-        l--;
-        check(&l, max);
-
-        if (l == r)
-            break;
-
-        //faces[l-1]->render();
-        nums.push(l);
-
-        r++;
-        check(&r, max);
-
-        if (l == r)
-            break;
-
-        //faces[r-1]->render();
-        nums.push(r);
-
-    }
-
-    while (!nums.empty()) {
-        faces[nums.top() - 1]->render();
-        nums.pop();
-    }
-}
-#endif // if 0
-
-
 void CarouselArrangement::set_current_face (int c, int r) 
 {
     current_face = c + (r * n_columns);
@@ -532,13 +492,19 @@ void CarouselArrangement::do_movement(void)
 
 }
 
-void CarouselArrangement::do_position_GL(void) 
+void CarouselArrangement::position_faces(void) 
 {
-    glTranslatef(0.0f,0.0f, z_offset);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
 
-    glRotatef(X_Rot, 1.0f, 0.0f, 0.0f);
+    //glRotatef(X_Rot, 1.0f, 0.0f, 0.0f);
+    extern Vector zero;  // rotate around the center (0,0,0)
+    camera->RotateAroundPoint(zero, -deg_to_rad(X_Rot), 1, 0, 0);
+
+    camera->Look();
 
     glRotatef(goto_face_val, 0.0f, 1.0f, 0.0f);
+    //camera->RotateAroundPoint(center, -deg_to_rad(goto_face_val), 0, 1, 0);
 }
 
 void CarouselArrangement::goto_random_face(void)
@@ -656,7 +622,8 @@ void CarouselArrangement::entry_exit_move(void)
 CylinderArrangement::CylinderArrangement(VDesktops &vdesks, 
 					 float fw, float fh, 
 					 FaceSet *fs,
-					 Config *c)
+					 Config *c,
+                                         Camera *cam)
     : Arrangement (fs, c)
 {
     two_face_gap = 3;
@@ -669,6 +636,8 @@ CylinderArrangement::CylinderArrangement(VDesktops &vdesks,
     face_width = fw;
     face_height = fh;
     
+    camera = cam;
+
     calculate_faces(vdesks);
     
     goto_face_mv.disable();
@@ -751,7 +720,7 @@ void CylinderArrangement::calculate_faces(VDesktops & vdesks)
         }
     }
 
-    make_display_list();
+    //make_display_list();
 
 }
 
@@ -852,11 +821,16 @@ void CylinderArrangement::do_movement(void)
         clamp_degrees(&goto_face_val);
 }
 
-void CylinderArrangement::do_position_GL(void) 
+void CylinderArrangement::position_faces(void) 
 {
-    glTranslatef(0.0f,0.0f, z_offset);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
 
-    glRotatef(X_Rot, 1.0f, 0.0f, 0.0f);
+    //glRotatef(X_Rot, 1.0f, 0.0f, 0.0f);
+    extern Vector zero;  // rotate around the center (0,0,0)
+    camera->RotateAroundPoint(zero, -deg_to_rad(X_Rot), 1, 0, 0);
+
+    camera->Look();
 
     glRotatef(goto_face_val, 0.0f, 1.0f, 0.0f);
 }
@@ -916,14 +890,16 @@ void CylinderArrangement::entry_exit_move(void)
 ViewMasterArrangement::ViewMasterArrangement(VDesktops &vdesks, 
                                              float fw, float fh,
                                              FaceSet *fs,
-                                             Config *c)
+                                             Config *c,
+                                             Camera *cam)
     : Arrangement (fs, c)
 {
     z_offset_distance = 10.0;
         
     face_width = fw;
     face_height = fh;
-                
+
+    camera = cam;
 
     calculate_faces(vdesks);
 
@@ -1102,13 +1078,26 @@ float ViewMasterArrangement::get_base_z_offset(void)
 }
 
 
-void ViewMasterArrangement::do_position_GL(void) 
+void ViewMasterArrangement::position_faces(void) 
 {
-    glTranslatef(0.0f,0.0f, z_offset);
-    glTranslatef(-center_distance, 0.0f, 0.0f);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
+    
+    //glTranslatef(-center_distance, 0.0f, 0.0f);
+    camera->position.x = center_distance;
+    camera->viewvector.x = center_distance;
+
+    camera->Look();
+
     glRotatef(goto_face_val, 0.0f, 0.0f, 1.0f);
 }
 
+void ViewMasterArrangement::render()
+{
+    //glCallList(face_list);
+    face_set->render_all();
+}
+
 void ViewMasterArrangement::do_movement(void) 
 {
     if (!goto_face_mv.at_destination())
@@ -1124,12 +1113,15 @@ void ViewMasterArrangement::do_movement(void)
 PriceIsRightArrangement::PriceIsRightArrangement(VDesktops &vdesks, 
                                                  float fw, float fh, 
                                                  FaceSet *fs,
-                                                 Config *c)
+                                                 Config *c, 
+                                                 Camera *cam)
     : Arrangement (fs, c)
 {
     face_width = fw;
     face_height = fh;
                 
+    camera = cam;
+
     calculate_faces(vdesks);
        
     goto_face_mv.disable();
@@ -1194,7 +1186,7 @@ void PriceIsRightArrangement::calculate_faces(VDesktops & vdesks)
         face_set->set_face (i, tl, tr, bl, br);
     }
 
-    make_display_list();
+    //make_display_list();
 
 }
 
@@ -1294,9 +1286,12 @@ void PriceIsRightArrangement::do_movement(void)
         clamp_degrees(&goto_face_val);
 }
 
-void PriceIsRightArrangement::do_position_GL(void) 
+void PriceIsRightArrangement::position_faces(void) 
 {
-    glTranslatef(0.0f,0.0f, z_offset);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
+
+    camera->Look();
 
     glRotatef(goto_face_val, 1.0f, 0.0f, 0.0f);
 }
@@ -1310,7 +1305,8 @@ void PriceIsRightArrangement::do_position_GL(void)
 LinearArrangement::LinearArrangement(VDesktops &vdesks, 
                                      float fw, float fh,
                                      FaceSet *fs,
-                                     Config *c)
+                                     Config *c,
+                                     Camera *cam)
     : Arrangement (fs, c)
 {
     z_offset_distance = 10.0;
@@ -1322,6 +1318,8 @@ LinearArrangement::LinearArrangement(VDesktops &vdesks,
     face_width = fw;
     face_height = fh;
 
+    camera = cam;
+
     calculate_faces(vdesks);
 
     x_trans_mv.disable();
@@ -1588,16 +1586,38 @@ float LinearArrangement::goto_row(int row)
 #endif // if 0
 
 
-void LinearArrangement::do_position_GL(void) 
+void LinearArrangement::position_faces(void) 
 {
     //if (spin_z_rot)
     //    glRotatef(spin_z_rot,0.0f,0.0f,1.0f);
 
-    glTranslatef(0.0f,0.0f, z_offset);
-    glTranslatef(X_Trans, 0.0f, 0.0f);
-    glTranslatef(0.0f, Y_Trans, 0.0f);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
+    
+    //glTranslatef(X_Trans, 0.0f, 0.0f);
+    camera->position.x = -X_Trans;
+    camera->viewvector.x = -X_Trans;
+
+    //glTranslatef(0.0f, Y_Trans, 0.0f);
+    camera->position.y = -Y_Trans;
+    camera->viewvector.y = -Y_Trans;
+
+    camera->Look();
 }
 
+void LinearArrangement::position_lighting(void)
+{
+    // don't do anything as this will keep the light center on our
+    // current view
+}
+
+void LinearArrangement::render()
+{
+    //glCallList(face_list);
+    face_set->render_all();
+}
+
+
 void LinearArrangement::do_movement(void) 
 {
     if (!x_trans_mv.at_destination()) {
@@ -1664,12 +1684,15 @@ void FlipArrangement::make_display_list(void)
 FlipArrangement::FlipArrangement(VDesktops & vdesks, 
                                  float fw, float fh,
                                  FaceSet *fs,
-                                 Config *c) 
+                                 Config *c,
+                                 Camera *cam)
     : Arrangement (fs, c)
 {
     face_width = fw;
     face_height = fh;
 
+    camera = cam;
+
     face_A = new Face (0);
     if (!face_A) {
         msgout(ERROR, "out of memory\n");
@@ -1750,7 +1773,7 @@ void FlipArrangement::calculate_faces(VDesktops & vdesks)
     if (!showing_face)
         showing_face = face_A;
 
-    msgout (DEBUG, "set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "set tid = %d\n", face_set->get_texture_id(current_face));
 
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 }
@@ -1758,7 +1781,7 @@ void FlipArrangement::calculate_faces(VDesktops & vdesks)
 // there is no entry exit for flips
 void FlipArrangement::entry_movement_start(void)
 {
-    msgout (DEBUG, "----------4 set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "----------4 set tid = %d\n", face_set->get_texture_id(current_face));
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 }
 
@@ -1782,7 +1805,7 @@ void FlipArrangement::set_current_face (int c, int r)
     current_col = c;
     current_row = r;
 
-    msgout (DEBUG, "---------- set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "---------- set tid = %d\n", face_set->get_texture_id(current_face));
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 
     goto_face_val = goto_face_mv.destination(); 
@@ -1801,9 +1824,13 @@ float FlipArrangement::get_base_z_offset(void)
 }
 
 
-void FlipArrangement::do_position_GL(void) 
+void FlipArrangement::position_faces(void) 
 {
-    glTranslatef(0.0f,0.0f, z_offset);
+    //glTranslatef(0.0f,0.0f, z_offset);
+    camera->position.z = -z_offset;
+
+    camera->Look();
+
     glRotatef(goto_face_val, 0.0f, 1.0f, 0.0f);
     if (spin_val) {
         glRotatef(spin_val,  1.0f,  1.0f,  1.0f);
@@ -1891,7 +1918,7 @@ float FlipArrangement::goto_face (desktop_coords_t dest)
         showing_face = face_A;
     }
 
-    msgout (DEBUG, "---------1 set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "---------1 set tid = %d\n", face_set->get_texture_id(current_face));
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 
     return cfg->options->face_change_steps;
@@ -1954,7 +1981,7 @@ void FlipArrangement::goto_right(void)
         showing_face = face_A;
     }
 
-    msgout (DEBUG, "----------2 set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "----------2 set tid = %d\n", face_set->get_texture_id(current_face));
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 }
 
@@ -1987,10 +2014,14 @@ void FlipArrangement::goto_left(void)
         showing_face = face_A;
     }
 
-    msgout (DEBUG, "----------3 set tid = %d\n", face_set->get_texture_id(current_face));
+    //msgout (DEBUG, "----------3 set tid = %d\n", face_set->get_texture_id(current_face));
     showing_face->set_texture_id(face_set->get_texture_id(current_face));
 }
 
 
+int FlipArrangement::allow_blending(void)
+{
+    return 0;  // flip looks bad with blending
+}
 
 #endif  // if !defined ONE_ARRANGEMENT_ONLY
