@@ -28,6 +28,7 @@
 #include "move.hpp" // for Movement
 #include "xutil.hpp"
 #include "config.hpp"
+#include "camera.hpp"
 
 
 // ===================================================================
@@ -75,6 +76,8 @@ protected:
 
     FaceSet *face_set;
 
+    Camera *camera;
+
     GLuint face_list;
 
     Movement goto_face_mv;
@@ -106,7 +109,8 @@ public:
     virtual float get_base_z_offset(void) = 0;
     virtual float get_distant_z_offset(void) = 0;
     virtual void do_movement(void) = 0;
-    virtual void do_position_GL(void) = 0;
+    virtual void position_faces(void) = 0;
+    virtual void position_lighting(void);
     virtual void render();
     virtual void make_display_list(void);
     virtual void set_current_face (int col, int row);
@@ -121,6 +125,8 @@ public:
     virtual int is_entry_movement_done(void);
     virtual int is_exit_movement_done(void);
 
+    virtual int allow_blending(void);
+
     // "locally" defined functions
     int get_current_column(void);
     int get_number_of_columns(void);
@@ -151,7 +157,7 @@ private:
 
 public:
     CarouselArrangement(VDesktops &vdesks, float fw, float fh, 
-			FaceSet *fs, Config *c);
+			FaceSet *fs, Config *c, Camera *cam);
 
     void calculate_faces(VDesktops & vdesks);
 
@@ -164,7 +170,7 @@ public:
     float get_base_z_offset(void); 
     float get_distant_z_offset(void); 
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
 
     // Entry/Exit movement functions
     void entry_movement_start(void);
@@ -193,7 +199,7 @@ private:
 
 public:
     CylinderArrangement(VDesktops &vdesks, float fw, float fh, 
-			FaceSet *fs, Config *c);
+			FaceSet *fs, Config *c, Camera *cam);
 
     void calculate_faces(VDesktops & vdesks);
 
@@ -206,7 +212,7 @@ public:
     float get_base_z_offset(void); 
     float get_distant_z_offset(void); 
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
 
     // Entry/Exit movement functions
     void entry_movement_start(void);
@@ -227,7 +233,7 @@ private:
 
 public:
     ViewMasterArrangement(VDesktops &vdesks, float fw, float fh,
-                          FaceSet *fs, Config *c);
+                          FaceSet *fs, Config *c, Camera *cam);
 
     void calculate_faces(VDesktops & vdesks);
 
@@ -240,7 +246,8 @@ public:
     float get_base_z_offset(void); 
     float get_distant_z_offset(void); 
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
+    void render();
 };
 
 
@@ -255,7 +262,7 @@ private:
     float radius;
 public:
     PriceIsRightArrangement(VDesktops &vdesks, float fw, float fh,
-                            FaceSet *fs, Config *c);
+                            FaceSet *fs, Config *c, Camera *cam);
 
     void calculate_faces(VDesktops & vdesks);
 
@@ -268,7 +275,7 @@ public:
     float get_base_z_offset(void); 
     float get_distant_z_offset(void); 
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
 };
 
 
@@ -287,7 +294,7 @@ private:
 
 public:
     LinearArrangement(VDesktops &vdesks, float fw, float fh,
-                      FaceSet *fs, Config *c);
+                      FaceSet *fs, Config *c, Camera *cam);
 
     void calculate_faces(VDesktops & vdesks);
     int get_type(void);
@@ -304,7 +311,9 @@ public:
 
     float goto_face(desktop_coords_t dest);
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
+    void position_lighting(void);
+    void render();
 
     // Entry/Exit movement functions which add spinning on exit
     void entry_exit_move(void);
@@ -330,7 +339,7 @@ private:
     int n_faces;
 public:
     FlipArrangement(VDesktops &vdesks, float fw, float fh,
-                    FaceSet *fs, Config *c);
+                    FaceSet *fs, Config *c, Camera *cam);
 
     ~FlipArrangement ();
 
@@ -346,7 +355,7 @@ public:
     float get_base_z_offset(void); 
     float get_distant_z_offset(void); 
     void do_movement(void);
-    void do_position_GL(void);
+    void position_faces(void);
 
     void goto_down(void);
     void goto_up(void);
@@ -355,6 +364,8 @@ public:
     void goto_right(void);
     void goto_left(void);
 
+    int allow_blending(void);
+
     // Entry/Exit movement functions
     void entry_movement_start(void);
     void exit_movement_start(void);
