@@ -64,7 +64,7 @@ public:
         memset (&bottomright, 0, sizeof(Point));
         memset (&normal, 0, sizeof(Point));
         texture_id = (GLuint)-1;
-        transparency = .5;
+        transparency = 1.0;
         active = 0;
         clist = (uint)-1;
         wireframe = wf;
@@ -138,16 +138,16 @@ public:
     }
 
     void render (void) {
-        int use_texture = 1;
 
         if (texture_id != (GLuint)-1) {
             glEnable(GL_TEXTURE_2D);
-            glBindTexture(GL_TEXTURE_2D, texture_id);   // 2d texture (x and y size)
+            glBindTexture(GL_TEXTURE_2D, texture_id);
 
-            //glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
-            //glEnable(GL_COLOR_MATERIAL);
+            glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
+            glEnable(GL_COLOR_MATERIAL);
 
-            glColor4f(transparency, transparency, transparency, transparency);
+            //glColor4f(transparency, transparency, transparency, transparency);
+            glColor4f(1.0, 1.0, 1.0, transparency);
 
             //msgout(DEBUG, "======== t=%f\n", transparency);
 
@@ -156,38 +156,36 @@ public:
         } else {
             // no digit or screenshot so show gray color
             glDisable(GL_TEXTURE_2D);
-            use_texture = 0;
-
 
             float v = 0.3;
 
             switch (cfg->options->frame_color) {
             case COLOR_RED:
-                glColor4f(v, 0, 0, 1.0); 
+                glColor4f(v, 0, 0, transparency); 
                 break;
             case COLOR_GREEN:
-                glColor4f(0, v, 0, 1.0);
+                glColor4f(0, v, 0, transparency);
                 break;
             case COLOR_BLUE:
-                glColor4f(0, 0, v, 1.0);
+                glColor4f(0, 0, v, transparency);
                 break;
             case COLOR_LIGHTBLUE:
-                glColor4f(0, v, v, 1.0);
+                glColor4f(0, v, v, transparency);
                 break;
             case COLOR_GRAY:
-                glColor4f(v, v, v, 1.0);
+                glColor4f(v, v, v, transparency);
                 break;
             case COLOR_WHITE:
-                glColor4f(1.0, 1.0, 1.0, 1.0);
+                glColor4f(1.0, 1.0, 1.0, transparency);
                 break;
             case COLOR_PURPLE:
-                glColor4f(v, 0, v, 1.0);
+                glColor4f(v, 0, v, transparency);
                 break;
             case COLOR_YELLOW:
-                glColor4f(v, v, 0, 1.0);
+                glColor4f(v, v, 0, transparency);
                 break;
             default: // gray
-                glColor4f(v, v, v, 1.0);
+                glColor4f(v, v, v, transparency);
                 break;
             }
 
@@ -199,6 +197,7 @@ public:
             glDisable (GL_LIGHTING);
         }
 
+
         if (clist != (uint)-1) {
             glCallList (clist);
         } else {
@@ -224,7 +223,6 @@ public:
 
         glEnable (GL_LIGHTING);
 
-
         // restore to fill
         glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     }
