@@ -1,6 +1,6 @@
 #!/bin/sh
 
-VERS=0.2.8
+VERS="0.2.9"
 
 ls | grep -v autom4te | grep -v CVS | sed s:^:3ddesktop-$VERS/: >MANIFEST
 cd ..
