@@ -79,6 +79,42 @@ void msgout (int level, char *msg, ...)
 }
 
 
+void 
+convert_imlib_image_to_opengl_data(int texture_size, Imlib_Image imlib_img, unsigned char *out_buff)
+{
+    imlib_context_set_image(imlib_img);
+    Imlib_Image imgfinal = imlib_create_cropped_scaled_image(0, 0, 
+                                            imlib_image_get_width(), 
+                                            imlib_image_get_height(),
+                                            texture_size,
+                                            texture_size);  
+
+    imlib_context_set_image(imgfinal);
+    unsigned int *tmp = imlib_image_get_data();
+
+    int w, h;
+    int x, y;
+    int offset;
+    int img_offset;
+
+    w = h = texture_size;
+    
+    for (y = 0; y < h; y++) {
+        for (x = 0; x < w; x++) {
+            offset = (y * w + x) * 4;
+            img_offset = (y * w + x);
+
+            out_buff[offset]     = (tmp[img_offset] >> 16) & 0xff;
+            out_buff[offset + 1] = (tmp[img_offset] >> 8) & 0xff;
+            out_buff[offset + 2] =  tmp[img_offset] & 0xff;
+            out_buff[offset + 3] = (tmp[img_offset] >> 24) & 0xff;
+        }
+    }
+    imlib_image_put_back_data(tmp);
+    imlib_free_image();
+}
+
+
 // quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  
 // See http://www.dcs.ed.ac.uk/~mxr/gfx/2d/BMP.txt for more info.
 int load_image(char *filename, 
@@ -97,6 +133,8 @@ int load_image(char *filename,
     
     unsigned char *working_buff = NULL;
 
+    unsigned int size_x, size_y;
+
     // make sure the file is there.
     if ((file = fopen(filename, "rb"))==NULL)
     {
@@ -111,22 +149,30 @@ int load_image(char *filename,
     fseek(file, 18, SEEK_CUR);
 
     // read the width
-    if ((i = fread(sizex, 4, 1, file)) != 1) {
+    if ((i = fread(&size_x, 4, 1, file)) != 1) {
 	msgout(ERROR,"Error reading width from %s.\n", filename);
 	return 0;
     }
-    *sizex = bswap_32(*sizex);
 
-    //msgout(DEBUG,"Width of %s: %u\n", filename, *sizex);
+    // expected size is passed it - fail if its not as expected
+    if (size_x != *sizex) {
+        msgout(DEBUG,"Error: width of %s is %u not %u\n", filename, size_x, *sizex);
+        return 0;
+    }
+    *sizex = bswap_32(size_x);
     
     // read the height 
-    if ((i = fread(sizey, 4, 1, file)) != 1) {
+    if ((i = fread(&size_y, 4, 1, file)) != 1) {
 	msgout(ERROR,"Error reading height from %s.\n", filename);
 	return 0;
     }
-    *sizey = bswap_32(*sizey);
 
-    //msgout(DEBUG,"Height of %s: %u\n", filename, *sizey);
+    // expected size is passed in - fail if its not as expected
+    if (size_y != *sizey) {
+        msgout(DEBUG,"Error: height of %s is %u not %u\n", filename, size_y, *sizey);
+        return 0;
+    }
+    *sizey = bswap_32(size_y);
     
     // read the planes
     if ((fread(&planes, 2, 1, file)) != 1) {
@@ -249,7 +295,7 @@ void context_switch(void)
     // sleep(0) doesn't seem to work
 
     tm.tv_sec = 0;
-    tm.tv_usec = 1;
+    tm.tv_usec = 100;
 
     select (0, 0, 0, 0, &tm); /* sleep */
 }
@@ -273,9 +319,8 @@ void sleep_ms (int ms )
 }
 
 #if 0
-// ------
-// String rendering routine; leverages on GLUT routine.
 
+// uses a glut routine
 void 
 ourPrintString(void *font, char *str)
 {
@@ -284,6 +329,7 @@ ourPrintString(void *font, char *str)
     for(i=0;i<l;i++)
         glutBitmapCharacter(font,*str++);
 } // END ourPrintString
+
 #endif // if 0
 
 
