@@ -21,7 +21,16 @@
 #ifndef _UTIL_H
 #define _UTIL_H
 #include <math.h>     // for cos
+#include <sys/param.h> // BSD 
+#ifdef BSD
+#include <sys/endian.h>
+#else
 #include <endian.h>
+#endif
+
+#include <X11/Xlib.h>
+#include <Imlib2.h>   // for Imlib_Image
+
 #include "3ddesk.h"
 
 #define MAX_FACES  30  // this is a lot
@@ -35,6 +44,7 @@ typedef float matrix_t[4][4];
 typedef float vertex_t[4];
 
 // prototypes
+void convert_imlib_image_to_opengl_data(int texture_size, Imlib_Image imlib_img, unsigned char *out_buff);
 int load_image(char *filename, unsigned char *data, 
                unsigned int *sizex, unsigned int *sizey, int add_alpha);
 void clamp_degrees (float *value);
