@@ -115,8 +115,16 @@ int GLWindow::open_display()
 
     screen = DefaultScreen(display);
 
-    screen_width = DisplayWidth(display, screen);
-    screen_height = DisplayHeight(display, screen);
+    if (cfg->screen_width) {
+        screen_width = cfg->screen_width;
+    } else {  // zero is default
+        screen_width = DisplayWidth(display, screen);
+    }
+    if (cfg->screen_height) { 
+        screen_height = cfg->screen_height;
+    } else {  // zero is default
+        screen_height = DisplayHeight(display, screen);
+    }
 
     Visual *vis = DefaultVisual(display, screen);
     Colormap cm = DefaultColormap(display, screen);
@@ -521,6 +529,7 @@ int GLWindow::grab_screenshot_data()
 {
     // deleted in destructor (life of object)
     if (!screenshot_data) {
+        msgout(DEBUG, "Allocating screenshot buffer %d bytes\n", cfg->texture_size * cfg->texture_size * 4);
         screenshot_data = new unsigned char [cfg->texture_size * cfg->texture_size * 4];
         if (screenshot_data == NULL) {
             msgout (ERROR, "out of memory\n");
@@ -530,44 +539,14 @@ int GLWindow::grab_screenshot_data()
 
     Imlib_Image img = imlib_create_image_from_drawable((Pixmap)0,0,0,screen_width,screen_height,1);
    
-    imlib_context_set_image(img);
-    Imlib_Image imgfinal = imlib_create_cropped_scaled_image(0, 0, 
-                                      screen_width, screen_height,
-                                                cfg->texture_size,
-                                                cfg->texture_size);  
-    imlib_context_set_image(imgfinal);	   
-    unsigned int *tmp = imlib_image_get_data();
+    convert_imlib_image_to_opengl_data(cfg->texture_size, img, screenshot_data);
 
-    int w = imlib_image_get_width();
-    int h = imlib_image_get_height();
-
-    int x, y;
-    int offset;
-    int img_offset;
-
-    w = h = cfg->texture_size;
-    
-    for (y = 0; y < h; y++) {
-
-        for (x = 0; x < w; x++) {
-
-            offset = (y * w + x) * 4;
-            img_offset = (y * w + x);
-
-            screenshot_data[offset]     = (tmp[img_offset] >> 16) & 0xff;
-            screenshot_data[offset + 1] = (tmp[img_offset] >> 8) & 0xff;
-            screenshot_data[offset + 2] =  tmp[img_offset] & 0xff;
-            screenshot_data[offset + 3] = (tmp[img_offset] >> 24) & 0xff;
-
-
-        }
-    }
-    imlib_free_image();			// saves ~ 10MB of res. Mem @1024x1024 texture_size
     imlib_context_set_image(img);
     imlib_free_image();
     return 0;
 }
 
+
 unsigned char *GLWindow::get_screenshot_data_ptr(void)
 {
     return screenshot_data;
